/**
 * This file is part of the PNML Tool project. The Universal PNML Tool is a tool that can be used for creating and modifying all kinds of Petri nets and for exporting them in PNML syntax. 
 * Copyright (C) 2009 YoujunHu, s061331@student.dtu.dk
 *  
 * PNML Tool is free software; you can redistribute it and/or modify it under the terms of the Eclipse Public License as published by Eclipse Foundation;version 1 of the License.
 * PNML Tool is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  Eclipse Public License for more details.
 * You should have received a copy of the Eclipse Public License along with PNML Tool, if not, write to the Eclipse Foundation, Inc.102 Centrepointe Drive Ottawa, Ontario, Canada, K2G 6B1 
 *
 * $Id$
 */
package org.pnml.tools.epnk.diagram.notationtransfer;

import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.emf.transaction.NotificationFilter;
import org.eclipse.gmf.runtime.notation.Bendpoints;
import org.eclipse.gmf.runtime.notation.Bounds;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.gmf.runtime.notation.Edge;
import org.eclipse.gmf.runtime.notation.LayoutConstraint;
import org.eclipse.gmf.runtime.notation.Location;
import org.eclipse.gmf.runtime.notation.NotationPackage;
import org.eclipse.gmf.runtime.notation.RelativeBendpoints;
import org.eclipse.gmf.runtime.notation.Size;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.gmf.runtime.notation.datatype.RelativeBendpoint;
import org.pnml.tools.epnk.diagram.part.PNMLCoreModelDiagramEditorPlugin;
import org.pnml.tools.epnk.pnmlcoremodel.AnnotationGraphics;
import org.pnml.tools.epnk.pnmlcoremodel.Arc;
import org.pnml.tools.epnk.pnmlcoremodel.ArcGraphics;
import org.pnml.tools.epnk.pnmlcoremodel.Coordinate;
import org.pnml.tools.epnk.pnmlcoremodel.Label;
import org.pnml.tools.epnk.pnmlcoremodel.LabelProxy;
import org.pnml.tools.epnk.pnmlcoremodel.Node;
import org.pnml.tools.epnk.pnmlcoremodel.NodeGraphics;
import org.pnml.tools.epnk.pnmlcoremodel.Object;
import org.pnml.tools.epnk.pnmlcoremodel.Page;
import org.pnml.tools.epnk.pnmlcoremodel.PageLabelProxy;
import org.pnml.tools.epnk.pnmlcoremodel.PnmlcoremodelFactory;



/**
 * The class updates Petri net graphics when the GMF diagram is changed. 
 * @generated NOT
 * @authors Lukasz, Youjun, eki
 */
public class GraphicElementListener extends EContentAdapter {

	private NotificationFilter ModifiedFilter;

	public GraphicElementListener() {
		ModifiedFilter = NotificationFilter.createEventTypeFilter(
				Notification.SET).or(
				NotificationFilter.createEventTypeFilter(Notification.UNSET))
				.or(NotificationFilter.createEventTypeFilter(
				Notification.ADD)).
				or(NotificationFilter.createEventTypeFilter(
						Notification.REMOVE));
	}

	public void notifyChanged(Notification notification) {
		if (notification.getNotifier() instanceof EObject) {
			super.notifyChanged(notification);
		}
		if (ModifiedFilter.matches(notification)
				&& (notification instanceof ENotificationImpl)
				&& (notification.getNotifier() instanceof EObject)
				) {
			
			ENotificationImpl eNotification = (ENotificationImpl) notification;
			EObject eNotifier = (EObject) eNotification.getNotifier();

			if (NotationPackage.eINSTANCE.getLocation().isSuperTypeOf(
					eNotifier.eClass())) {
				updateNodeBounds(eNotification, eNotifier);
			} else if (eNotifier instanceof RelativeBendpoints) {
				// Deal with bendpoints of arcs
				EObject object = eNotifier.eContainer();
				if (object instanceof Edge) {
					Edge edge = (Edge) object;
					object = edge.getElement();
					if (object != null && object instanceof Arc) {
						Arc arc = (Arc) object;
						this.updateIntermediatePositions(edge, arc);
					}
				}
			} else if (eNotifier instanceof org.eclipse.gmf.runtime.notation.Node && eNotification.getNewValue() != null &&
					eNotification.getNewValue() instanceof org.eclipse.gmf.runtime.notation.Connector) {
				// this makes sure that the position of the ProxyLabel is propagated to the Label, 
				// when it is connected to a node.
				org.eclipse.gmf.runtime.notation.Node node = (org.eclipse.gmf.runtime.notation.Node) eNotifier;
				EObject element = node.getElement();
				if (element instanceof LabelProxy) {
					View view = (View) eNotifier;
					updateNodeBounds(view, element);
				}
				
			}
		}
	}

	private void updateIntermediatePositions(Edge edge, Arc arc) {
		ArcGraphics arcGraphics = (ArcGraphics) arc.getGraphics();
		if (arcGraphics == null) {
			arcGraphics = PnmlcoremodelFactory.eINSTANCE.createArcGraphics();
			arc.setGraphics(arcGraphics);
		}
		
		arcGraphics.getPosition().clear();
		int x = 0;
		int y = 0;
		Node source = arc.getSource();
		if (source != null) {
			// FIXME should be more defensive
			NodeGraphics nodeGraphics = (NodeGraphics) source.getGraphics();
			if (nodeGraphics != null) {
				x = (int) nodeGraphics.getPosition().getX();
				y = (int) nodeGraphics.getPosition().getY();
			}
		}
		
		Bendpoints bendpoints = edge.getBendpoints();
		if (bendpoints instanceof RelativeBendpoints) {
			// FIXME should be more defensive
			RelativeBendpoints relativeBendpoints = (RelativeBendpoints) bendpoints;
			for (java.lang.Object pointObj: relativeBendpoints.getPoints() ) {
				if (pointObj instanceof RelativeBendpoint) {
					RelativeBendpoint point = (RelativeBendpoint) pointObj;
					Coordinate coord = PnmlcoremodelFactory.eINSTANCE.createCoordinate();
					coord.setX(x + point.getSourceX());
					coord.setY(y + point.getSourceY());
					arcGraphics.getPosition().add(coord);
				}
			}
		}
	}
	
	private void updateNodeBounds(ENotificationImpl eNotification,
			EObject eNotifier) {
		View view = (View) eNotifier.eContainer();
		EObject modelElement = view.getElement();
	    updateNodeBounds(view, modelElement);
	}

	private void updateNodeBounds(View view, EObject modelElement) {
		if (view instanceof org.eclipse.gmf.runtime.notation.Node) {
			org.eclipse.gmf.runtime.notation.Node viewNode = (org.eclipse.gmf.runtime.notation.Node) view;

			if ((modelElement instanceof Node || modelElement instanceof Page)){
				Object modelNode = (Object) modelElement;

				NodeGraphics nodeGraphics = (NodeGraphics) modelNode.getGraphics();
				if (nodeGraphics == null){
					nodeGraphics = PnmlcoremodelFactory.eINSTANCE.createNodeGraphics();
					modelNode.setGraphics(nodeGraphics);
				}

				Coordinate newPosition = getNodePosition(viewNode);
				nodeGraphics.setPosition(newPosition);

				Coordinate newDimension = getNodeDimension(viewNode);
				nodeGraphics.setDimension(newDimension);

				// The labels attached to this node also need to be updated too, since their positions
				// are relative to this position! 

				EObject container = viewNode.eContainer();
				if (container != null && container instanceof Diagram) {
					Diagram diagram = (Diagram) container;
					for (java.lang.Object child: diagram.getChildren()) {
						if (child instanceof org.eclipse.gmf.runtime.notation.Node) {
							org.eclipse.gmf.runtime.notation.Node node = 
								(org.eclipse.gmf.runtime.notation.Node) child;
							EObject element = node.getElement();
							if (element != null && element instanceof LabelProxy) {
								Label label = ((LabelProxy) element).getLabel();
								if (label != null && label.eContainer() == modelElement);
								updateNodeBounds(node,element);
							} 
						}
					}
				}

				// the intermediate points of an arc also need to be updated

				for (java.lang.Object child: viewNode.getSourceEdges()) {
					if (child instanceof Edge) {
						Edge edge = (Edge) child;
						EObject element = edge.getElement();
						if (element != null && element instanceof Arc) {
							updateIntermediatePositions(edge, (Arc) element);
						}
					}
				}


			} else if (modelElement instanceof LabelProxy) {
				LabelProxy labelProxy = (LabelProxy) modelElement;
				Label label = labelProxy.getLabel();

				if (label != null) {
					EObject container = label.eContainer();
					if (container instanceof Object) {
						Object object = (Object) container;
						AnnotationGraphics labelGraphics = (AnnotationGraphics) label.getGraphics();
						if (labelGraphics == null){
							labelGraphics = PnmlcoremodelFactory.eINSTANCE.createAnnotationGraphics();
							label.setGraphics(labelGraphics);
						}

						Coordinate newPosition = getLabelOffset(
								object, 
								viewNode);
						labelGraphics.setOffset(newPosition);
					}
				}
			} else if (modelElement instanceof PageLabelProxy) {
				PageLabelProxy pageLabelProxy = (PageLabelProxy) modelElement;
				Label label = pageLabelProxy.getLabel();

				if (label != null) {
					AnnotationGraphics labelGraphics = (AnnotationGraphics) label.getGraphics();
					if (labelGraphics == null){
						labelGraphics = PnmlcoremodelFactory.eINSTANCE.createAnnotationGraphics();
						label.setGraphics(labelGraphics);
					}

					Coordinate newPosition = getAbsolutePNMLPosition(viewNode);
					labelGraphics.setOffset(newPosition);
				}
			} else {
				Status status = new Status(Status.ERROR, 
						PNMLCoreModelDiagramEditorPlugin.ID, 
						Status.OK, 
						"Undealt case in graphics update: should not happen!", 
						new Exception());
				PNMLCoreModelDiagramEditorPlugin.getInstance().getLog().log(status);
			}
		}
	}
	
	private static Coordinate getNodePosition(org.eclipse.gmf.runtime.notation.Node node) {
		Coordinate coord = PnmlcoremodelFactory.eINSTANCE.createCoordinate();

		LayoutConstraint layout = node.getLayoutConstraint();
		if (layout != null) {
			if (layout instanceof Bounds) {
				Bounds bounds = (Bounds) layout;
				// Transform from upper left corner to center
				coord.setX(bounds.getX()+bounds.getWidth()/2.0f);
				coord.setY(bounds.getY()+bounds.getHeight()/2.0f);
			} else if (layout instanceof Location) {
				// Transform from upper left corner to center
                // with GMF standard with (40)
				Location location = (Location) layout;
				coord.setX(location.getX()+20.0f);
				coord.setY(location.getY()+20.0f);
			}
		}

		return coord;
	}
	
	private static Coordinate getNodeDimension(org.eclipse.gmf.runtime.notation.Node node) {
		Coordinate coord = PnmlcoremodelFactory.eINSTANCE.createCoordinate();

		LayoutConstraint layout = node.getLayoutConstraint();
		if (layout != null) {
			if (layout instanceof Size) {
				Size size = (Size) layout;
				int width=size.getWidth();
				if (width <0) width = 40;
				coord.setX(width);
				int height = size.getHeight();
				if (height<0) height=40;
				coord.setY(height);
			} 
		}

		return coord;
	}
	
	private static Coordinate getLabelOffset(Object object, org.eclipse.gmf.runtime.notation.Node node) {
		Coordinate coord = PnmlcoremodelFactory.eINSTANCE.createCoordinate();

		LayoutConstraint layout = node.getLayoutConstraint();
		if (layout != null) {
			if (layout instanceof Location) {
				Location labelLocation = (Location) layout;
				Coordinate absoluteObjectPosition = NotationModelHelper.getAbsolutePNMLPositionObject(object);
				coord.setX(labelLocation.getX() - absoluteObjectPosition.getX());
				coord.setY(labelLocation.getY() - absoluteObjectPosition.getY());
			} 
		}

		return coord;
	}
	
	private static Coordinate getAbsolutePNMLPosition(org.eclipse.gmf.runtime.notation.Node node) {
		Coordinate coord = PnmlcoremodelFactory.eINSTANCE.createCoordinate();

		LayoutConstraint layout = node.getLayoutConstraint();
		if (layout != null) {
			if (layout instanceof Location) {
				Location labelLocation = (Location) layout;
				coord.setX(labelLocation.getX());
				coord.setY(labelLocation.getY());
			} 
		}

		return coord;
	}
	

}
